/**---------------------------------------------------------------------------------------------
* \file				avagoMinipodCtrl.h
*
* \brief This file contains the user driver to control the 6 AVAGO miniPOD of the AMC40 board.
* 
* \author PYD : Pierre-Yves Duval
* \version 0.1
* \date 17/4/2012

-------------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------------------------
* PYD : 17/4/2012 initial version
* PYD : 12/6/2015 Several i2c bus implemented for PCIe40
*-----------------------------------------------------------------------------------------------
*/

#define MPOD_DBG  0 /*!<debug flag */
#define MPOD_TEST 0 /*!<test flag */

#define MPOD_ERRORSTAT        2 /*!<index of data nor ready, intl, LOS, fault status*/
#define MPOD_LOSREG1          9 /*!<index of LOS reg for channel  11 downto 8 (reg low quarter)*/
#define MPOD_LOSREG2         10 /*!<index of LOS reg for channel 7 downto 0 */
#define MPOD_FAULTREG1       11 /*!<index of TX FAULT reg for channel  11 downto 8 (reg low quarter)*/
#define MPOD_FAULTREG2       12 /*!<index of TX FAULT reg for channel 7 downto 0 */
#define MPOD_INTEMPMSB       28 /*!<index of internal temperature monitor MSB*/
#define MPOD_INTEMPLSB       29 /*!<index of internal temperature monitor LSB*/
#define MPOD_33VCCMSB	     32 /*!<index of 3.3 Vcc monitor MSB*/
#define MPOD_33VCCLSB	     33 /*!<index of 3.3 Vcc monitor LSB*/
#define MPOD_25VCCMSB	     34 /*!<index of 2.5 Vcc monitor MSB*/
#define MPOD_25VCCLSB	     35 /*!<index of 2.5 Vcc monitor LSB*/

/* FOR TX component ONLY on page OOh */
#define MPOD_BIASCURMSB11   40  /*!<index of TX Current BIAS monitor MSB fot channel 11*/
#define MPOD_BIASCURLSB11   41  /*!<index of TX Current BIAS monitor LSB fot channel 11*/
#define MPOD_BIASCURMSB10   42  /*!<index of TX Current BIAS monitor MSB fot channel 10*/
#define MPOD_BIASCURLSB10   43  /*!<index of TX Current BIAS monitor LSB fot channel 10*/
#define MPOD_BIASCURMSB09   44  /*!<index of TX Current BIAS monitor MSB fot channel 09*/
#define MPOD_BIASCURLSB09   45  /*!<index of TX Current BIAS monitor LSB fot channel 09*/
#define MPOD_BIASCURMSB08   46  /*!<index of TX Current BIAS monitor MSB fot channel 08*/
#define MPOD_BIASCURLSB08   47  /*!<index of TX Current BIAS monitor LSB fot channel 08*/
#define MPOD_BIASCURMSB07   48  /*!<index of TX Current BIAS monitor MSB fot channel 07*/
#define MPOD_BIASCURLSB07   49  /*!<index of TX Current BIAS monitor LSB fot channel 07*/
#define MPOD_BIASCURMSB06   50  /*!<index of TX Current BIAS monitor MSB fot channel 06*/
#define MPOD_BIASCURLSB06   51  /*!<index of TX Current BIAS monitor LSB fot channel 06*/
#define MPOD_BIASCURMSB05   52  /*!<index of TX Current BIAS monitor MSB fot channel 05*/
#define MPOD_BIASCURLSB05   53  /*!<index of TX Current BIAS monitor LSB fot channel 05*/
#define MPOD_BIASCURMSB04   54  /*!<index of TX Current BIAS monitor MSB fot channel 04*/
#define MPOD_BIASCURLSB04   55  /*!<index of TX Current BIAS monitor LSB fot channel 04*/
#define MPOD_BIASCURMSB03   56  /*!<index of TX Current BIAS monitor MSB fot channel 03*/
#define MPOD_BIASCURLSB03   57  /*!<index of TX Current BIAS monitor LSB fot channel 03*/
#define MPOD_BIASCURMSB02   58  /*!<index of TX Current BIAS monitor MSB fot channel 02*/
#define MPOD_BIASCURLSB02   59  /*!<index of TX Current BIAS monitor LSB fot channel 02*/
#define MPOD_BIASCURMSB01   60  /*!<index of TX Current BIAS monitor MSB fot channel 01*/
#define MPOD_BIASCURLSB01   61  /*!<index of TX Current BIAS monitor LSB fot channel 01*/
#define MPOD_BIASCURMSB00   62  /*!<index of TX Current BIAS monitor MSB fot channel 00*/
#define MPOD_BIASCURLSB00   63  /*!<index of TX Current BIAS monitor LSB fot channel 00*/

#define MPOD_LIGHTOUTMSB11   64  /*!<index of TX Light Output monitor MSB fot channel 11*/
#define MPOD_LIGHTOUTLSB11   65  /*!<index of TX Light Output monitor LSB fot channel 11*/
#define MPOD_LIGHTOUTMSB10   66  /*!<index of TX Light Output monitor MSB fot channel 10*/
#define MPOD_LIGHTOUTLSB10   67  /*!<index of TX Light Output monitor LSB fot channel 10*/
#define MPOD_LIGHTOUTMSB09   68  /*!<index of TX Light Output monitor MSB fot channel 09*/
#define MPOD_LIGHTOUTLSB09   69  /*!<index of TX Light Output monitor LSB fot channel 09*/
#define MPOD_LIGHTOUTMSB08   70  /*!<index of TX Light Output monitor MSB fot channel 08*/
#define MPOD_LIGHTOUTLSB08   71  /*!<index of TX Light Output monitor LSB fot channel 08*/
#define MPOD_LIGHTOUTMSB07   72  /*!<index of TX Light Output monitor MSB fot channel 07*/
#define MPOD_LIGHTOUTLSB07   73  /*!<index of TX Light Output monitor LSB fot channel 07*/
#define MPOD_LIGHTOUTMSB06   74  /*!<index of TX Light Output monitor MSB fot channel 06*/
#define MPOD_LIGHTOUTLSB06   75  /*!<index of TX Light Output monitor LSB fot channel 06*/
#define MPOD_LIGHTOUTMSB05   76  /*!<index of TX Light Output monitor MSB fot channel 05*/
#define MPOD_LIGHTOUTLSB05   77  /*!<index of TX Light Output monitor LSB fot channel 05*/
#define MPOD_LIGHTOUTMSB04   78  /*!<index of TX Light Output monitor MSB fot channel 04*/
#define MPOD_LIGHTOUTLSB04   79  /*!<index of TX Light Output monitor LSB fot channel 04*/
#define MPOD_LIGHTOUTMSB03   80  /*!<index of TX Light Output monitor MSB fot channel 03*/
#define MPOD_LIGHTOUTLSB03   81  /*!<index of TX Light Output monitor LSB fot channel 03*/
#define MPOD_LIGHTOUTMSB02   82  /*!<index of TX Light Output monitor MSB fot channel 02*/
#define MPOD_LIGHTOUTLSB02   83  /*!<index of TX Light Output monitor LSB fot channel 02*/
#define MPOD_LIGHTOUTMSB01   84  /*!<index of TX Light Output monitor MSB fot channel 01*/
#define MPOD_LIGHTOUTLSB01   85  /*!<index of TX Light Output monitor LSB fot channel 01*/
#define MPOD_LIGHTOUTMSB00   86  /*!<index of TX Light Output monitor MSB fot channel 00*/
#define MPOD_LIGHTOUTLSB00   87  /*!<index of TX Light Output monitor LSB fot channel 00*/

/* FOR RX component ONLY on page OOh */
#define MPOD_OPPOWERMSB11   64  /*!<index of RX optical input optical power monitor MSB fot channel 11*/
#define MPOD_OPPOWERLSB11   65  /*!<index of RX optical input optical power monitor LSB fot channel 11*/
#define MPOD_OPPOWERMSB10   66  /*!<index of RX optical input optical power monitor MSB fot channel 10*/
#define MPOD_OPPOWERLSB10   67  /*!<index of RX optical input optical power monitor LSB fot channel 10*/
#define MPOD_OPPOWERMSB09   68  /*!<index of RX optical input optical power monitor MSB fot channel 09*/
#define MPOD_OPPOWERLSB09   69  /*!<index of RX optical input optical power monitor LSB fot channel 09*/
#define MPOD_OPPOWERMSB08   70  /*!<index of RX optical input optical power monitor MSB fot channel 08*/
#define MPOD_OPPOWERLSB08   71  /*!<index of RX optical input optical power monitor LSB fot channel 08*/
#define MPOD_OPPOWERMSB07   72  /*!<index of RX optical input optical power monitor MSB fot channel 07*/
#define MPOD_OPPOWERLSB07   73  /*!<index of RX optical input optical power monitor LSB fot channel 07*/
#define MPOD_OPPOWERMSB06   74  /*!<index of RX optical input optical power monitor MSB fot channel 06*/
#define MPOD_OPPOWERLSB06   75  /*!<index of RX optical input optical power monitor LSB fot channel 06*/
#define MPOD_OPPOWERMSB05   76  /*!<index of RX optical input optical power monitor MSB fot channel 05*/
#define MPOD_OPPOWERLSB05   77  /*!<index of RX optical input optical power monitor LSB fot channel 05*/
#define MPOD_OPPOWERMSB04   78  /*!<index of RX optical input optical power monitor MSB fot channel 04*/
#define MPOD_OPPOWERLSB04   79  /*!<index of RX optical input optical power monitor LSB fot channel 04*/
#define MPOD_OPPOWERMSB03   80  /*!<index of RX optical input optical power monitor MSB fot channel 03*/
#define MPOD_OPPOWERLSB03   81  /*!<index of RX optical input optical power monitor LSB fot channel 03*/
#define MPOD_OPPOWERMSB02   82  /*!<index of RX optical input optical power monitor MSB fot channel 02*/
#define MPOD_OPPOWERLSB02   83  /*!<index of RX optical input optical power monitor LSB fot channel 02*/
#define MPOD_OPPOWERMSB01   84  /*!<index of RX optical input optical power monitor MSB fot channel 01*/
#define MPOD_OPPOWERLSB01   85  /*!<index of RX optical input optical power monitor LSB fot channel 01*/
#define MPOD_OPPOWERMSB00   86  /*!<index of RX optical input optical power monitor MSB fot channel 00*/
#define MPOD_OPPOWERLSB00   87  /*!<index of RX optical input optical power monitor LSB fot channel 00*/

#define MPOD_ELAPSEREGMSB    88  /*!<index of elapsed operating time MSB in 2 hours units*/
#define MPOD_ELAPSEREGLSB    88  /*!<index of elapsed operating time LSB in 2 hours units*/
#define MPOD_SQUELCHREG1     94  /*!<index of squelch reg for channel  11 downto 8 (reg low quarter)*/
#define MPOD_SQUELCHREG2     95  /*!<index of squelch reg for channel 7 downto 0 */
#define MPOD_PAGEREG        127  /*!<page selection register */         
#define MPOD_VENDORNAMEREG  152  /*!<index of the register with the first byte of the name of the vendor in ASCII*/
#define MPOD_VENDOROUIREG   168  /*!<index of the register with the first byte of the OUI as hexa values*/
#define MPOD_VENDORPARTREG  171  /*!<index of the register with the first byte of the part number of the vendor in ASCII*/
#define MPOD_VENDORREVREG   187  /*!<index of the register with the first byte of the revision number of the vendor in ASCII*/
#define MPOD_VENDORSERREG   189  /*!<index of the register with the first byte of the serial number of the vendor in ASCII*/
#define MPOD_VENDORDATEREG  205  /*!<index of the register with the first byte of the date of the vendor in ASCII*/

/* for TX and RX on page 00h */
#define MPOD_RESET           91  /*!<index of register to reset all registers to factory values */
#define MPOD_CHDISABLE1      92  /*!<index of DISABLE reg for channel 11 downto 8 (reg low quarter)*/
#define MPOD_CHDISABLE2      93  /*!<index of DISABLE reg for channel 7 downto 0 */
//PYD
/* for TX and RX on page 01h */
#define MPOD_POLARFLIP1      226  /*!<index of DISABLE reg for channel 11 downto 8 (reg low quarter)*/
#define MPOD_POLARFLIP2      227  /*!<index of DISABLE reg for channel 7 downto 0 */
//PYD
/* for TX only on page 00h */
#define MPOD_MARGINACT1      99  /*!<index of Margin Activation reg for channel 11 downto 8 (reg low quarter)*/
#define MPOD_MARGINACT2     100  /*!<index of Margin Activation reg for channel 7 downto 0 */

/* FOR RX component ONLY on page O1h */
#define MPOD_RXVOD6      228  /*!<index of RX output amplitude control VOD for channels 11 and 10*/
#define MPOD_RXVOD5      229  /*!<index of RX output amplitude control VOD for channels 9 and 8*/
#define MPOD_RXVOD4      230  /*!<index of RX output amplitude control VOD for channels 7 and 6*/
#define MPOD_RXVOD3      231  /*!<index of RX output amplitude control VOD for channels 5 and 4*/
#define MPOD_RXVOD2      232  /*!<index of RX output amplitude control VOD for channels 3 and 2*/
#define MPOD_RXVOD1      233  /*!<index of RX output amplitude control VOD for channels 1 and 0*/
/* FOR TX component ONLY on page O1h */
#define MPOD_TXEQUALZ6      228  /*!<index of RX output equalization control for channels 11 and 10*/
#define MPOD_TXEQUALZ5      229  /*!<index of RX output equalization control for channels 9 and 8*/
#define MPOD_TXEQUALZ4      230  /*!<index of RX output equalization control for channels 7 and 6*/
#define MPOD_TXEQUALZ3      231  /*!<index of RX output equalization control for channels 5 and 4*/
#define MPOD_TXEQUALZ2      232  /*!<index of RX output equalization control for channels 3 and 2*/
#define MPOD_TXEQUALZ1      233  /*!<index of RX output equalization control for channels 1 and 0*/

/* FOR RX component ONLY on page O1h */
#define MPOD_RXDEAMPH6   234  /*!<index of RX output deamphasis control for channels 11 and 10*/
#define MPOD_RXDEAMPH5   235  /*!<index of RX output deamphasis control for channels 9 and 8*/
#define MPOD_RXDEAMPH4   236  /*!<index of RX output deamphasis control for channels 7 and 6*/
#define MPOD_RXDEAMPH3   237  /*!<index of RX output deamphasis control for channels 5 and 4*/
#define MPOD_RXDEAMPH2   238  /*!<index of RX output deamphasis control for channels 3 and 2*/
#define MPOD_RXDEAMPH1   239  /*!<index of RX output deamphasis control for channels 1 and 0*/


#define MPOD_NBVENDORNAME     15 /*!<size in register of the vendor name "AVAGO"*/
#define MPOD_NBVENDOROUI       3 /*!<size in register of the vendor OUI (IEEE ID) "OOh 17h 6Ah" for AVAGO*/
#define MPOD_NBVENDORPART     16 /*!<size in register of the vendor part number in ascii "AFBR-811FN1Z"*/
#define MPOD_NBVENDORREV       2 /*!<size in register of the vendor revision number in ASCII: coded with space (20h)*/
#define MPOD_NBVENDORSER      16 /*!<size in register of the vendor serial number in ASCII*/
#define MPOD_NBVENDORDATE      8 /*!<size in register of the vendor date code YYYYMMDD (ASCII)*/

#ifdef MPOCPP
extern "C" { 
#endif

/* public functions */
int mpod_setMode			  		(int mode);
int mpod_setTextMode				(int mode);
int mpod_exists               (int mpid);
int mpod_init                 (int dev, unsigned inCLK, unsigned SCLfreq);
void mpod_getInit             (int dev, unsigned *inCLK, unsigned *SCLfreq);
int mpod_isTX                 (int mpid);
int mpod_isRX                 (int mpid);
int mpod_dumpErrStatus        (int dev, int mpid);
int mpod_dumpErrStatusAll(int dev);
int mpod_dumpVendorData       (int dev, int mpid);
int mpod_getVendorName        (int dev, int mpid, char **bloc);
int mpod_getVendorOUI         (int dev, int mpid, char **bloc);
int mpod_getVendorPartNumber  (int dev, int mpid, char **bloc);
int mpod_getVendorRevNumber   (int dev, int mpid, char **bloc);
int mpod_getVendorDate        (int dev, int mpid, char **bloc);
int mpod_getVendorSerialNumber(int dev, int mpid, char **bloc);
int mpod_squelchDisable       (int dev, int mpid , int channel , int disable);
int mpod_squelchDisableAll    (int dev, int mpid, int disable);
int mpod_squelchDumpAll       (int dev, int mpid);
int mpod_losDumpAll           (int dev, int mpid);
int mpod_faultDumpAll         (int dev, int mpid);
int mpod_internalTemp         (int dev, int mpid, int *temp);
int mpod_internalTempDumpAll(int dev);
int mpod_internal33Vcc        (int dev, int mpid, int *vcc);
int mpod_internal33VccDumpAll(int dev);
int mpod_internal25Vcc        (int dev, int mpid, int *vcc);
int mpod_internal25VccDumpAll(int dev);
int mpod_biasCurrent          (int dev, int mpid, float **data);
int mpod_getBiasCurrent       (int dev, int mpid, float *data);
int mpod_lightOutput          (int dev, int mpid, float **data);
int mpod_getLightOutput       (int dev, int mpid, float *data);
int mpod_lightInput           (int dev, int mpid, float **data);
int mpod_getlightInput        (int dev, int mpid, float *data);
int mpod_elapsedPowerOn       (int dev, int mpid, int *dur);
int mpod_reset                (int dev, int mpid);
int mpod_channelDisable       (int dev, int mpid, int channel, int flag);
int mpod_channelDisDumpAll    (int dev, int mpid);
//PYD
int mpod_polarityFlip         (int dev, int mpid, int channel, int flag);
int mpod_channelPolarFlipDumpAll(int dev, int mpid);
//PYD
int mpod_marginActivation     (int dev, int mpid, int channel, int flag);
int mpod_marginActDumpAll     (int dev, int mpid);

int mpod_read                 (int dev, int mpid, char reg, unsigned *val);
int mpod_readVOD              (int dev, int mpid, int **data);
int mpod_getVOD					(int dev, int mpid, int *data);
int mpod_readVODSingle        (int dev, int mpid, int channel, int *code);
int mpod_writeVOD             (int dev, int mpid, int *data);
int mpod_writeVODSingle       (int dev, int mpid, int channel, int code);

int mpod_readDeamphasis       (int dev, int mpid, int **data);
int mpod_getDeamphasis			(int dev, int mpid, int *data);
int mpod_readDeamphasisSingle (int dev, int mpid, int channel, int *code);
int mpod_writeDeamphasis      (int dev, int mpid, int *data);
int mpod_writeDeamphasisSingle(int dev, int mpid, int channel, int code);

int mpod_readEqualization       (int dev, int mpid, int **data);
int mpod_getEqualization        (int dev, int mpid, int *data);
int mpod_readEqualizationSingle (int dev, int mpid, int channel, int *code);
int mpod_writeEqualization      (int dev, int mpid, int *data);
int mpod_writeEqualizationSingle(int dev, int mpid, int channel, int code);

#ifdef MPOCPP
} 
#endif
